import type { ErrorPayload } from "vite/types/hmrPayload";
export declare function createController(options: {
    getIframe: () => HTMLIFrameElement | null;
    listener(event: PreviewEvent): void;
}): PreviewIframeController;
export interface PreviewIframeController {
    start(): void;
    stop(): void;
    showLoading(): void;
    loadComponent(options: LoadComponentOptions): void;
}
export interface LoadComponentOptions {
    filePath: string;
    componentName: string;
    variantKey: string | null;
    customVariantPropsSource: string;
    defaultPropsSource: string;
}
export declare type PreviewEvent = PreviewBootstrapped | BeforeRender | PreviewUpdate | Action | LogMessage;
export declare type PreviewBootstrapped = {
    kind: "bootstrapped";
};
export declare type BeforeRender = {
    kind: "before-render";
};
export declare type PreviewUpdate = {
    kind: "update";
    viteError: ErrorPayload | null;
    rendering: {
        kind: "error";
        error: string;
    } | {
        kind: "success";
        variantKey: string;
        variants: Variant[];
    } | null;
};
export interface Action {
    kind: "action";
    type: "fn" | "url";
    path: string;
}
export interface LogMessage {
    kind: "log-message";
    timestamp: number;
    level: LogLevel;
    message: string;
}
export declare type LogLevel = "log" | "info" | "warn" | "error";
export interface Variant {
    key: string;
    label: string;
    isEditorDriven?: boolean;
}
export declare type RendererLoader = (options: {
    wrapperModule: any;
    wrapperName?: string;
    componentFilePath: string;
    componentModule: any;
    componentName?: string;
}) => Promise<{
    variants: Array<Variant & {
        props?: any;
    }>;
    render: (props: any) => Promise<void>;
}>;
