"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateDefaultProps = void 0;
const type_analyzer_1 = require("@previewjs/type-analyzer");
const generate_value_1 = require("./generate-value");
const EMPTY_SET = new Set();
/**
 * Generates props that will be set on a component even if no props are specified.
 *
 * In particular, this is useful for callbacks like `onClick` which don't need to be set explicitly.
 */
function generateDefaultProps(propsType, types) {
    propsType = resolveType(propsType);
    if (propsType.kind !== "object") {
        return {
            source: "{}",
            propKeys: EMPTY_SET,
        };
    }
    const propKeys = new Set();
    let text = "";
    text += "{";
    for (const [propertyName, propertyType] of Object.entries(propsType.fields)) {
        const resolvedPropertyType = resolveType(propertyType);
        if (resolvedPropertyType.kind === "function") {
            propKeys.add(propertyName);
            const returnType = resolvedPropertyType.returnType;
            text += `"${propertyName.replace(/"/g, '\\"')}": fn("${propertyName.replace(/"/g, '\\"')}", ${(0, generate_value_1.generateValue)(returnType, types, [], [], true)}),`;
        }
    }
    text += "}";
    return {
        source: text,
        propKeys,
    };
    function resolveType(type) {
        if (type.kind === "name") {
            const resolved = types[type.name];
            if (resolved) {
                return (0, type_analyzer_1.evaluateType)(resolved, []);
            }
        }
        return type;
    }
}
exports.generateDefaultProps = generateDefaultProps;
