"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateInvocation = void 0;
const type_analyzer_1 = require("@previewjs/type-analyzer");
const prettier_1 = __importDefault(require("prettier"));
const generate_value_1 = require("./generate-value");
/**
 * Generates an invocation source for a specific component.
 *
 * Example:
 * ```
 * properties = { title: "foo" }
 * ```
 */
function generateInvocation(propsType, providedKeys, collected) {
    [propsType] = (0, type_analyzer_1.dereferenceType)(propsType, collected, []);
    if (propsType.kind === "object") {
        propsType = (0, type_analyzer_1.objectType)(Object.fromEntries(Object.entries(propsType.fields).filter(([fieldName]) => !providedKeys.has(fieldName))));
    }
    let value = (0, generate_value_1.generateValue)(propsType, collected, [], [], false);
    if (value === "undefined") {
        value = "{}";
    }
    const source = `properties = ${value}`;
    return prettier_1.default
        .format(source, {
        filepath: "component.js",
    })
        .trim();
}
exports.generateInvocation = generateInvocation;
