"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const type_analyzer_1 = require("@previewjs/type-analyzer");
const generate_type_declarations_1 = require("./generate-type-declarations");
describe("generateTypeDeclarations", () => {
    test("simple props with object type", () => {
        expect((0, generate_type_declarations_1.generateTypeDeclarations)("MyComponent", (0, type_analyzer_1.objectType)({
            foo: type_analyzer_1.STRING_TYPE,
        }), new Set([]), {})).toMatchSnapshot();
    });
    test("simple props with named type", () => {
        expect((0, generate_type_declarations_1.generateTypeDeclarations)("MyComponent", (0, type_analyzer_1.namedType)("/foo.tsx:MyComponentProps"), new Set([]), {
            "/foo.tsx:MyComponentProps": {
                type: (0, type_analyzer_1.objectType)({
                    foo: type_analyzer_1.STRING_TYPE,
                }),
                parameters: {},
            },
        })).toMatchSnapshot();
        expect((0, generate_type_declarations_1.generateTypeDeclarations)("MyComponent", (0, type_analyzer_1.namedType)("/foo.tsx:Foo"), new Set([]), {
            "/foo.tsx:Foo": {
                type: (0, type_analyzer_1.objectType)({
                    foo: type_analyzer_1.STRING_TYPE,
                }),
                parameters: {},
            },
        })).toMatchSnapshot();
    });
    test("recursive named type", () => {
        expect((0, generate_type_declarations_1.generateTypeDeclarations)("MyComponent", (0, type_analyzer_1.namedType)("/foo.tsx:Foo"), new Set([]), {
            "/foo.tsx:Foo": {
                type: (0, type_analyzer_1.objectType)({
                    child: (0, type_analyzer_1.optionalType)((0, type_analyzer_1.namedType)("/foo.tsx:Foo")),
                }),
                parameters: {},
            },
        })).toMatchSnapshot();
    });
    test("recursive function type", () => {
        expect((0, generate_type_declarations_1.generateTypeDeclarations)("MyComponent", (0, type_analyzer_1.namedType)("/foo.tsx:MyComponentProps"), new Set([]), {
            "/foo.tsx:MyComponentProps": {
                type: (0, type_analyzer_1.objectType)({
                    foo: (0, type_analyzer_1.namedType)("/foo.tsx:Fn"),
                }),
                parameters: {},
            },
            "/foo.tsx:Fn": {
                type: (0, type_analyzer_1.functionType)((0, type_analyzer_1.optionalType)((0, type_analyzer_1.namedType)("/foo.tsx:Fn"))),
                parameters: {},
            },
        })).toMatchSnapshot();
    });
    test("all types", () => {
        expect((0, generate_type_declarations_1.generateTypeDeclarations)("MyComponent", (0, type_analyzer_1.namedType)("/foo.tsx:Foo"), new Set([]), {
            "/foo.tsx:Foo": {
                type: (0, type_analyzer_1.objectType)({
                    anyType: type_analyzer_1.ANY_TYPE,
                    unknownType: type_analyzer_1.UNKNOWN_TYPE,
                    neverType: type_analyzer_1.NEVER_TYPE,
                    voidType: type_analyzer_1.VOID_TYPE,
                    nullType: type_analyzer_1.NULL_TYPE,
                    booleanType: type_analyzer_1.BOOLEAN_TYPE,
                    stringType: type_analyzer_1.STRING_TYPE,
                    numberType: type_analyzer_1.NUMBER_TYPE,
                    reactNodeType: type_analyzer_1.NODE_TYPE,
                    numberLiteral: (0, type_analyzer_1.literalType)(123),
                    stringLiteral: (0, type_analyzer_1.literalType)("foo"),
                    trueLiteral: (0, type_analyzer_1.literalType)(true),
                    falseLiteral: (0, type_analyzer_1.literalType)(false),
                    stringEnumType: (0, type_analyzer_1.enumType)({
                        a: "A",
                        b: "B",
                        c: "C",
                    }),
                    numberEnumType: (0, type_analyzer_1.enumType)({
                        a: 3,
                        b: 2,
                        c: 1,
                    }),
                    arrayType: (0, type_analyzer_1.arrayType)(type_analyzer_1.STRING_TYPE),
                    setType: (0, type_analyzer_1.setType)(type_analyzer_1.STRING_TYPE),
                    recordType: (0, type_analyzer_1.recordType)(type_analyzer_1.STRING_TYPE, type_analyzer_1.NUMBER_TYPE),
                    unionType: (0, type_analyzer_1.unionType)([type_analyzer_1.STRING_TYPE, type_analyzer_1.NUMBER_TYPE]),
                    intersectionType: (0, type_analyzer_1.intersectionType)([type_analyzer_1.STRING_TYPE, type_analyzer_1.NUMBER_TYPE]),
                    functionType: (0, type_analyzer_1.functionType)(type_analyzer_1.STRING_TYPE),
                    promiseType: (0, type_analyzer_1.promiseType)(type_analyzer_1.STRING_TYPE),
                    namedType: (0, type_analyzer_1.namedType)("/foo.tsx:Bar"),
                }),
                parameters: {},
            },
            "/foo.tsx:Bar": {
                type: (0, type_analyzer_1.objectType)({
                    bar: (0, type_analyzer_1.functionType)(type_analyzer_1.STRING_TYPE),
                }),
                parameters: {},
            },
        })).toMatchSnapshot();
    });
    test("duplicate types in different files", () => {
        expect((0, generate_type_declarations_1.generateTypeDeclarations)("MyComponent", (0, type_analyzer_1.namedType)("/foo.tsx:Foo"), new Set([]), {
            "/foo.tsx:Foo": {
                type: (0, type_analyzer_1.objectType)({
                    foo: (0, type_analyzer_1.namedType)("/bar.tsx:Foo"),
                }),
                parameters: {},
            },
            "/bar.tsx:Foo": {
                type: (0, type_analyzer_1.objectType)({
                    bar: (0, type_analyzer_1.namedType)("/baz.tsx:Foo"),
                }),
                parameters: {},
            },
            "/baz.tsx:Foo": {
                type: (0, type_analyzer_1.objectType)({
                    baz: type_analyzer_1.STRING_TYPE,
                }),
                parameters: {},
            },
        })).toMatchSnapshot();
    });
    test("optional props in object type", () => {
        expect((0, generate_type_declarations_1.generateTypeDeclarations)("MyComponent", (0, type_analyzer_1.objectType)({
            a: type_analyzer_1.STRING_TYPE,
            b: (0, type_analyzer_1.optionalType)(type_analyzer_1.STRING_TYPE),
            c: type_analyzer_1.ANY_TYPE,
            d: type_analyzer_1.UNKNOWN_TYPE,
            e: type_analyzer_1.STRING_TYPE,
            f: (0, type_analyzer_1.optionalType)(type_analyzer_1.STRING_TYPE),
        }), new Set([]), {})).toMatchSnapshot();
    });
    test("generic types", () => {
        expect((0, generate_type_declarations_1.generateTypeDeclarations)("MyComponent", (0, type_analyzer_1.namedType)("/foo.tsx:A"), new Set([]), {
            "/foo.tsx:A": {
                type: (0, type_analyzer_1.namedType)("T"),
                parameters: { T: (0, type_analyzer_1.namedType)("/foo.tsx:B") },
            },
            "/foo.tsx:B": {
                type: (0, type_analyzer_1.objectType)({
                    foo: (0, type_analyzer_1.namedType)("/foo.tsx:C", [(0, type_analyzer_1.namedType)("/foo.tsx:B")]),
                }),
                parameters: {},
            },
            "/foo.tsx:C": {
                type: (0, type_analyzer_1.objectType)({
                    t: (0, type_analyzer_1.namedType)("T"),
                    s: (0, type_analyzer_1.namedType)("S"),
                }),
                parameters: {
                    T: null,
                    S: (0, type_analyzer_1.namedType)("T"),
                },
            },
        })).toMatchSnapshot();
    });
});
