"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const type_analyzer_1 = require("@previewjs/type-analyzer");
const generate_value_1 = require("./generate-value");
describe("generateValue", () => {
    test("simple props with object type", () => {
        expect((0, generate_value_1.generateValue)((0, type_analyzer_1.objectType)({
            foo: type_analyzer_1.STRING_TYPE,
        }), {}, [], [], false)).toMatchInlineSnapshot(`
      "{
      foo: \\"foo\\",

      }"
    `);
    });
    test("simple props with named type", () => {
        expect((0, generate_value_1.generateValue)((0, type_analyzer_1.namedType)("/foo.tsx:MyComponentProps"), {
            "/foo.tsx:MyComponentProps": {
                type: (0, type_analyzer_1.objectType)({
                    foo: type_analyzer_1.STRING_TYPE,
                }),
                parameters: {},
            },
        }, [], [], false)).toMatchInlineSnapshot(`
      "{
      foo: \\"foo\\",

      }"
    `);
    });
    test("recursive named type", () => {
        expect((0, generate_value_1.generateValue)((0, type_analyzer_1.namedType)("/foo.tsx:Foo"), {
            "/foo.tsx:Foo": {
                type: (0, type_analyzer_1.objectType)({
                    child: (0, type_analyzer_1.namedType)("/foo.tsx:Bar"),
                }),
                parameters: {},
            },
            "/foo.tsx:Bar": {
                type: (0, type_analyzer_1.objectType)({
                    child: (0, type_analyzer_1.optionalType)((0, type_analyzer_1.namedType)("/foo.tsx:Foo")),
                }),
                parameters: {},
            },
        }, [], [], false)).toMatchInlineSnapshot(`
      "{
      child: {

      },

      }"
    `);
    });
    test("recursive function type", () => {
        expect((0, generate_value_1.generateValue)((0, type_analyzer_1.namedType)("/foo.tsx:MyComponentProps"), {
            "/foo.tsx:MyComponentProps": {
                type: (0, type_analyzer_1.objectType)({
                    foo: (0, type_analyzer_1.namedType)("/foo.tsx:Fn"),
                }),
                parameters: {},
            },
            "/foo.tsx:Fn": {
                type: (0, type_analyzer_1.functionType)((0, type_analyzer_1.namedType)("/foo.tsx:Fn")),
                parameters: {},
            },
        }, [], [], false)).toMatchInlineSnapshot(`
      "{
      foo: () => {},

      }"
    `);
    });
    test("self-returning type", () => {
        expect((0, generate_value_1.generateValue)((0, type_analyzer_1.namedType)("/foo.tsx:User"), {
            "/foo.tsx:User": {
                type: (0, type_analyzer_1.objectType)({
                    target: (0, type_analyzer_1.namedType)("/foo.tsx:User"),
                    friends: (0, type_analyzer_1.functionType)((0, type_analyzer_1.arrayType)((0, type_analyzer_1.namedType)("/foo.tsx:User"))),
                }),
                parameters: {},
            },
        }, [], [], false)).toMatchInlineSnapshot(`
      "{
      friends: () => ([]),

      }"
    `);
    });
    test("all types", () => {
        expect((0, generate_value_1.generateValue)((0, type_analyzer_1.namedType)("/foo.tsx:Foo"), {
            "/foo.tsx:Foo": {
                type: (0, type_analyzer_1.objectType)({
                    anyType: type_analyzer_1.ANY_TYPE,
                    unknownType: type_analyzer_1.UNKNOWN_TYPE,
                    neverType: type_analyzer_1.NEVER_TYPE,
                    voidType: type_analyzer_1.VOID_TYPE,
                    nullType: type_analyzer_1.NULL_TYPE,
                    booleanType: type_analyzer_1.BOOLEAN_TYPE,
                    stringType: type_analyzer_1.STRING_TYPE,
                    numberType: type_analyzer_1.NUMBER_TYPE,
                    reactNodeType: type_analyzer_1.NODE_TYPE,
                    numberLiteral: (0, type_analyzer_1.literalType)(123),
                    stringLiteral: (0, type_analyzer_1.literalType)("foo"),
                    trueLiteral: (0, type_analyzer_1.literalType)(true),
                    falseLiteral: (0, type_analyzer_1.literalType)(false),
                    stringEnumType: (0, type_analyzer_1.enumType)({
                        a: "A",
                        b: "B",
                        c: "C",
                    }),
                    numberEnumType: (0, type_analyzer_1.enumType)({
                        a: 3,
                        b: 2,
                        c: 1,
                    }),
                    arrayType: (0, type_analyzer_1.arrayType)(type_analyzer_1.STRING_TYPE),
                    setType: (0, type_analyzer_1.setType)(type_analyzer_1.STRING_TYPE),
                    recordType: (0, type_analyzer_1.recordType)(type_analyzer_1.STRING_TYPE, type_analyzer_1.NUMBER_TYPE),
                    unionType: (0, type_analyzer_1.unionType)([type_analyzer_1.STRING_TYPE, type_analyzer_1.NUMBER_TYPE]),
                    intersectionType: (0, type_analyzer_1.intersectionType)([type_analyzer_1.STRING_TYPE, type_analyzer_1.NUMBER_TYPE]),
                    functionType: (0, type_analyzer_1.functionType)(type_analyzer_1.STRING_TYPE),
                    promiseType: (0, type_analyzer_1.promiseType)(type_analyzer_1.STRING_TYPE),
                    namedType: (0, type_analyzer_1.namedType)("/foo.tsx:Bar"),
                }),
                parameters: {},
            },
            "/foo.tsx:Bar": {
                type: (0, type_analyzer_1.objectType)({
                    bar: (0, type_analyzer_1.functionType)(type_analyzer_1.STRING_TYPE),
                }),
                parameters: {},
            },
        }, [], [], false)).toMatchInlineSnapshot(`
      "{
      nullType: null,
      booleanType: false,
      stringType: \\"stringType\\",
      numberType: 100,
      reactNodeType: \\"reactNodeType\\",
      numberLiteral: 123,
      stringLiteral: \\"foo\\",
      trueLiteral: true,
      falseLiteral: false,
      stringEnumType: \\"A\\",
      numberEnumType: 3,
      arrayType: [\\"arrayType\\"],
      setType: new Set([\\"setType\\"]),
      recordType: {},
      unionType: \\"unionType\\",
      intersectionType: \\"intersectionType\\",
      functionType: () => (\\"functionType\\"),
      promiseType: Promise.reject(),
      namedType: {
      bar: () => (\\"namedType.bar\\"),

      },

      }"
    `);
    });
    test("optional props in object type", () => {
        expect((0, generate_value_1.generateValue)((0, type_analyzer_1.namedType)("/foo.tsx:Foo"), {
            "/foo.tsx:Foo": {
                type: (0, type_analyzer_1.objectType)({
                    a: type_analyzer_1.STRING_TYPE,
                    b: (0, type_analyzer_1.optionalType)(type_analyzer_1.STRING_TYPE),
                    c: type_analyzer_1.ANY_TYPE,
                    d: type_analyzer_1.UNKNOWN_TYPE,
                    e: type_analyzer_1.STRING_TYPE,
                    f: (0, type_analyzer_1.optionalType)(type_analyzer_1.STRING_TYPE),
                }),
                parameters: {},
            },
        }, [], [], false)).toMatchInlineSnapshot(`
      "{
      a: \\"a\\",
      e: \\"e\\",

      }"
    `);
    });
    test("missing named type", () => {
        expect((0, generate_value_1.generateValue)((0, type_analyzer_1.namedType)("/foo.tsx:Foo"), {
            "/foo.tsx:Foo": {
                type: (0, type_analyzer_1.objectType)({
                    a: (0, type_analyzer_1.namedType)("/foo.tsx:Missing"),
                }),
                parameters: {},
            },
        }, [], [], false)).toMatchInlineSnapshot(`
      "{

      }"
    `);
    });
    test("generic types", () => {
        expect((0, generate_value_1.generateValue)((0, type_analyzer_1.namedType)("/foo.tsx:A"), {
            "/foo.tsx:A": {
                type: (0, type_analyzer_1.namedType)("/foo.tsx:C", [(0, type_analyzer_1.namedType)("T")]),
                parameters: { T: type_analyzer_1.NUMBER_TYPE },
            },
            "/foo.tsx:B": {
                type: (0, type_analyzer_1.objectType)({
                    foo: (0, type_analyzer_1.namedType)("/foo.tsx:C", [(0, type_analyzer_1.namedType)("/foo.tsx:B")]),
                }),
                parameters: {},
            },
            "/foo.tsx:C": {
                type: (0, type_analyzer_1.objectType)({
                    t: (0, type_analyzer_1.namedType)("T"),
                    s: (0, type_analyzer_1.namedType)("S"),
                }),
                parameters: {
                    T: null,
                    S: (0, type_analyzer_1.namedType)("T"),
                },
            },
        }, [], [], false)).toMatchInlineSnapshot(`
      "{
      t: 100,
      s: 100,

      }"
    `);
    });
});
