"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadPreviewEnv = void 0;
const config_1 = require("@previewjs/config");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
async function loadPreviewEnv({ rootDirPath, setupEnvironment, frameworkPluginFactories, }) {
    const previewEnv = await setupEnvironment({ rootDirPath });
    let frameworkPlugin = await (0, config_1.readConfig)(rootDirPath).frameworkPlugin;
    fallbackToDefault: if (!frameworkPlugin) {
        const dependencies = await extractPackageDependencies(rootDirPath);
        for (const candidate of frameworkPluginFactories || []) {
            if (await candidate.isCompatible(dependencies)) {
                frameworkPlugin = await candidate.create();
                break fallbackToDefault;
            }
        }
        return null;
    }
    return {
        previewEnv,
        frameworkPlugin,
    };
}
exports.loadPreviewEnv = loadPreviewEnv;
async function extractPackageDependencies(rootDirPath) {
    const packageJsonPath = path_1.default.join(rootDirPath, "package.json");
    if (!(await fs_extra_1.default.pathExists(packageJsonPath))) {
        return {};
    }
    let { dependencies, devDependencies } = JSON.parse(await fs_extra_1.default.readFile(packageJsonPath, "utf8"));
    const allDependencies = {
        ...dependencies,
        ...devDependencies,
    };
    return Object.fromEntries(Object.entries(allDependencies).map(([name, version]) => {
        let majorVersion;
        if (typeof version !== "string") {
            majorVersion = 0;
        }
        else if (version.startsWith("^") || version.startsWith("~")) {
            majorVersion = parseInt(version.slice(1));
        }
        else {
            majorVersion = parseInt(version);
        }
        return [name, { majorVersion }];
    }));
}
