"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
const express_1 = __importDefault(require("express"));
const http_terminator_1 = require("http-terminator");
class Server {
    constructor(options) {
        this.serverTerminator = null;
        this.server = null;
        const app = (0, express_1.default)();
        app.use((_req, res, next) => {
            // Disable caching.
            // This helps ensure that we don't end up with issues such as when
            // assets are updated, or a new version of Preview.js is used.
            res.setHeader("Cache-Control", "max-age=0, must-revalidate");
            next();
        });
        for (const middleware of options.middlewares) {
            app.use(middleware);
        }
        this.app = app;
    }
    async start(port) {
        return new Promise((resolve) => {
            const server = (this.server = this.app.listen(port, () => {
                console.log(`Preview.js Server running at http://localhost:${port}.`);
                resolve(server);
            }));
            this.serverTerminator = (0, http_terminator_1.createHttpTerminator)({
                server: this.server,
                gracefulTerminationTimeout: 0,
            });
        });
    }
    async stop() {
        if (this.serverTerminator) {
            await this.serverTerminator.terminate();
            this.serverTerminator = null;
        }
        if (this.server) {
            const server = this.server;
            await new Promise((resolve) => server.close(resolve));
            this.server = null;
        }
        console.log(`Preview.js Server stopped.`);
    }
}
exports.Server = Server;
