"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.componentLoaderPlugin = void 0;
const url_1 = require("url");
const COMPONENT_LOADER_MODULE = "/@component-loader.js";
function componentLoaderPlugin(options) {
    return {
        name: "previewjs:component-loader",
        resolveId: async function (id) {
            if (id.startsWith(COMPONENT_LOADER_MODULE)) {
                return id;
            }
            return null;
        },
        load: async function (id) {
            if (!id.startsWith(COMPONENT_LOADER_MODULE)) {
                return null;
            }
            const params = new url_1.URLSearchParams(id.split("?")[1] || "");
            return generateComponentLoaderModule(params, options.config.wrapper);
        },
    };
}
exports.componentLoaderPlugin = componentLoaderPlugin;
function generateComponentLoaderModule(urlParams, wrapper) {
    const filePath = urlParams.get("p");
    const componentName = urlParams.get("c");
    if (filePath === null || componentName === null) {
        throw new Error(`Invalid use of /@component-loader.js module`);
    }
    const componentModuleId = `/${filePath.replace(/\\/g, "/")}`;
    return `import { updateComponent } from '/__previewjs_internal__/update-component';
import { load } from '/__previewjs_internal__/renderer/index';

let counter = 0;
export async function refresh() {
  const currentCounter = ++counter;
  let loadingError = null;
  ${wrapper
        ? `
  let wrapperModulePromise;
  if (import.meta.hot.data.preloadedWrapperModule) {
    wrapperModulePromise = Promise.resolve(import.meta.hot.data.preloadedWrapperModule);
  } else {
    wrapperModulePromise = import("/${wrapper.path}");
  }
  const wrapperModule = await wrapperModulePromise.catch(e => {
    console.error(e);
    loadingError = e.stack || e.message || null;
    return null;
  });
  `
        : `
  const wrapperModule = null;
  `}
  let componentModulePromise;
  if (import.meta.hot.data.preloadedComponentModule) {
    componentModulePromise = Promise.resolve(import.meta.hot.data.preloadedComponentModule);
  } else {
    componentModulePromise = import("${componentModuleId}");
  }
  const componentModule = await componentModulePromise.catch(e => {
    console.error(e);
    loadingError = e.stack || e.message || null;
    return null;
  });
  if (currentCounter !== counter) {
    // Abort to avoid double rendering.
    return;
  }
  await updateComponent({
    wrapperModule,
    wrapperName: ${JSON.stringify((wrapper === null || wrapper === void 0 ? void 0 : wrapper.componentName) || null)},
    componentModule,
    componentFilePath: ${JSON.stringify(filePath)},
    componentName: ${JSON.stringify(componentName)},
    loadingError,
    load,
  })
}

import.meta.hot.accept();

${wrapper
        ? `
import.meta.hot.accept(["${wrapper.path}"], ([wrapperModule]) => {
  import.meta.hot.data.preloadedWrapperModule = wrapperModule;
  refresh();
});
`
        : ``}

import.meta.hot.accept(["${componentModuleId}"], ([componentModule]) => {
  import.meta.hot.data.preloadedComponentModule = componentModule;
  refresh();
});
`;
}
