"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.cssModulesWithoutSuffixPlugin = void 0;
const path_1 = __importDefault(require("path"));
const extensions = new Set([".js", ".jsx", ".ts", ".tsx"]);
const importRegexp = /import\s+([\S]+)\s+from\s+('|")([\S]+)\.(css|less|sass|scss|styl|stylus|postcss)(\?[\S]*)?&?('|")/g;
/**
 * Allows imports of CSS modules without a .module.css suffix.
 *
 * See https://github.com/fwouts/previewjs/discussions/39
 */
function cssModulesWithoutSuffixPlugin() {
    return {
        name: "previewjs:css-modules-without-suffix",
        async transform(code, id) {
            // Ignore query parameters.
            id = id.split("?")[0];
            if (!extensions.has(path_1.default.extname(id))) {
                return;
            }
            return code.replace(importRegexp, 'import $1 from "$3.$4?$5&.module.css"');
        },
    };
}
exports.cssModulesWithoutSuffixPlugin = cssModulesWithoutSuffixPlugin;
