"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ViteManager = void 0;
const express_1 = __importDefault(require("express"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const recrawl_1 = require("recrawl");
const rollup_plugin_friendly_type_imports_1 = __importDefault(require("rollup-plugin-friendly-type-imports"));
const strip_ansi_1 = __importDefault(require("strip-ansi"));
const tsconfig_loader_js_1 = require("tsconfig-paths/lib/tsconfig-loader.js");
const vite = __importStar(require("vite"));
const vite_tsconfig_paths_1 = __importDefault(require("vite-tsconfig-paths"));
const component_loader_plugin_1 = require("./plugins/component-loader-plugin");
const css_modules_without_suffix_plugin_1 = require("./plugins/css-modules-without-suffix-plugin");
const virtual_plugin_1 = require("./plugins/virtual-plugin");
class ViteManager {
    constructor(options) {
        this.options = options;
        this.lastPingTimestamp = 0;
        const router = express_1.default.Router();
        router.get("/preview/", async (req, res) => {
            const template = await fs_extra_1.default.readFile(this.options.shadowHtmlFilePath, "utf-8");
            if (!this.viteServer) {
                res.status(404).end(`Vite is not running.`);
                return;
            }
            res
                .status(200)
                .set({ "Content-Type": "text/html" })
                .end(await this.viteServer.transformIndexHtml(req.originalUrl, template));
        });
        router.use("/ping", async (req, res) => {
            this.lastPingTimestamp = Date.now();
            res.json(JSON.stringify({
                pong: "match!",
            }));
        });
        router.use(async (req, res, next) => {
            const waitSeconds = 60;
            const waitUntil = Date.now() + waitSeconds * 1000;
            while (!this.viteServer) {
                await new Promise((resolve) => setTimeout(resolve, 50));
                if (Date.now() > waitUntil) {
                    throw new Error(`Vite server is not running after ${waitSeconds} seconds.`);
                }
            }
            this.viteServer.middlewares(req, res, next);
        });
        this.middleware = router;
    }
    async start(server, port) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q;
        // Find valid tsconfig.json files.
        //
        // Useful when the project may contain some invalid files.
        const typeScriptConfigFilePaths = await (0, recrawl_1.recrawl)({
            only: ["tsconfig.json"],
            skip: ["node_modules", ".git"],
        })(this.options.rootDirPath);
        const validTypeScriptFilePaths = [];
        for (const configFilePath of typeScriptConfigFilePaths) {
            try {
                (0, tsconfig_loader_js_1.loadTsconfig)(configFilePath);
                validTypeScriptFilePaths.push(configFilePath);
            }
            catch (e) {
                console.warn(`Encountered an invalid tsconfig.json file, ignoring: ${configFilePath}`);
            }
        }
        let tsInferredAlias = {};
        if (typeScriptConfigFilePaths.includes("tsconfig.json")) {
            // If there is a top-level tsconfig.json, use it to infer aliases.
            // While this is also done by vite-tsconfig-paths, it doesn't apply to CSS Modules and so on.
            const config = (0, tsconfig_loader_js_1.loadTsconfig)("tsconfig.json");
            if (((_a = config === null || config === void 0 ? void 0 : config.compilerOptions) === null || _a === void 0 ? void 0 : _a.baseUrl) && ((_b = config === null || config === void 0 ? void 0 : config.compilerOptions) === null || _b === void 0 ? void 0 : _b.paths)) {
                const { baseUrl, paths } = config.compilerOptions;
                for (const [match, mapping] of Object.entries(paths)) {
                    const firstMapping = mapping[0];
                    if (!firstMapping) {
                        continue;
                    }
                    const matchNoWildcard = match.endsWith("/*")
                        ? match.slice(0, match.length - 2)
                        : match;
                    const firstMappingNoWildcard = firstMapping.endsWith("/*")
                        ? firstMapping.slice(0, firstMapping.length - 2)
                        : firstMapping;
                    tsInferredAlias[matchNoWildcard] = path_1.default.join(this.options.rootDirPath, baseUrl, firstMappingNoWildcard);
                }
            }
        }
        const alias = {
            ...tsInferredAlias,
            ...this.options.config.alias,
            ...(_d = (_c = this.options.config.vite) === null || _c === void 0 ? void 0 : _c.resolve) === null || _d === void 0 ? void 0 : _d.alias,
        };
        const defaultLogger = vite.createLogger(this.options.logLevel);
        const frameworkPluginViteConfig = this.options.frameworkPlugin.viteConfig({
            ...this.options.config,
            alias,
        });
        const vitePlugins = [
            (0, vite_tsconfig_paths_1.default)({
                root: this.options.rootDirPath,
                projects: validTypeScriptFilePaths,
            }),
            (0, virtual_plugin_1.virtualPlugin)({
                reader: this.options.reader,
                rootDirPath: this.options.rootDirPath,
                moduleGraph: () => { var _a; return ((_a = this.viteServer) === null || _a === void 0 ? void 0 : _a.moduleGraph) || null; },
                esbuildOptions: frameworkPluginViteConfig.esbuild || {},
            }),
            (0, rollup_plugin_friendly_type_imports_1.default)({
                readFile: (absoluteFilePath) => this.options.reader.read(absoluteFilePath).then((entry) => {
                    if ((entry === null || entry === void 0 ? void 0 : entry.kind) !== "file") {
                        return null;
                    }
                    return entry.read();
                }),
            }),
            (0, css_modules_without_suffix_plugin_1.cssModulesWithoutSuffixPlugin)(),
            (0, component_loader_plugin_1.componentLoaderPlugin)({
                config: this.options.config,
            }),
            ...(frameworkPluginViteConfig.plugins || []),
            ...(((_e = this.options.config.vite) === null || _e === void 0 ? void 0 : _e.plugins) || []),
        ];
        // We need to patch handleHotUpdate() in every plugin because, by
        // default, HmrContext has a read() method that reads directly from
        // the file system. We want it to read from our reader, which could
        // be using an in-memory version instead.
        const plugins = vitePlugins.flat().map((plugin) => {
            if (!plugin || Array.isArray(plugin) || !plugin.handleHotUpdate) {
                return plugin;
            }
            const handleHotUpdate = plugin.handleHotUpdate.bind(plugin);
            return {
                ...plugin,
                handleHotUpdate: async (ctx) => {
                    return handleHotUpdate({
                        ...ctx,
                        read: async () => {
                            const entry = await this.options.reader.read(ctx.file);
                            if ((entry === null || entry === void 0 ? void 0 : entry.kind) !== "file") {
                                // Fall back to default behaviour.
                                return ctx.read();
                            }
                            return entry.read();
                        },
                    });
                },
            };
        });
        this.viteServer = await vite.createServer({
            ...frameworkPluginViteConfig,
            ...this.options.config.vite,
            configFile: false,
            root: this.options.rootDirPath,
            base: "/preview/",
            server: {
                middlewareMode: true,
                hmr: {
                    overlay: false,
                    server,
                    clientPort: port,
                    ...(typeof ((_g = (_f = this.options.config.vite) === null || _f === void 0 ? void 0 : _f.server) === null || _g === void 0 ? void 0 : _g.hmr) === "object"
                        ? (_j = (_h = this.options.config.vite) === null || _h === void 0 ? void 0 : _h.server) === null || _j === void 0 ? void 0 : _j.hmr
                        : {}),
                },
                ...(_k = this.options.config.vite) === null || _k === void 0 ? void 0 : _k.server,
            },
            customLogger: {
                info: defaultLogger.info,
                warn: defaultLogger.warn,
                error: (message, options) => {
                    var _a, _b;
                    defaultLogger.error(message, options);
                    const errorMessage = ((_a = options === null || options === void 0 ? void 0 : options.error) === null || _a === void 0 ? void 0 : _a.stack)
                        ? options.error.stack
                        : (0, strip_ansi_1.default)(message);
                    (_b = this.viteServer) === null || _b === void 0 ? void 0 : _b.ws.send({
                        type: "error",
                        err: {
                            message: errorMessage,
                            stack: "",
                        },
                    });
                },
                warnOnce: defaultLogger.warnOnce,
                clearScreen: () => { },
                hasWarned: defaultLogger.hasWarned,
                hasErrorLogged: defaultLogger.hasErrorLogged,
            },
            clearScreen: false,
            cacheDir: ((_l = this.options.config.vite) === null || _l === void 0 ? void 0 : _l.cacheDir) || this.options.cacheDir,
            publicDir: ((_m = this.options.config.vite) === null || _m === void 0 ? void 0 : _m.publicDir) || this.options.config.publicDir,
            plugins,
            define: {
                __filename: undefined,
                __dirname: undefined,
                ...frameworkPluginViteConfig.define,
                ...(_o = this.options.config.vite) === null || _o === void 0 ? void 0 : _o.define,
            },
            resolve: {
                ...(_p = this.options.config.vite) === null || _p === void 0 ? void 0 : _p.resolve,
                alias: {
                    "~": "",
                    "@": "",
                    ...alias,
                    ...(_q = frameworkPluginViteConfig.resolve) === null || _q === void 0 ? void 0 : _q.alias,
                },
            },
        });
    }
    async stop() {
        if (this.viteServer) {
            await this.viteServer.close();
            delete this.viteServer;
        }
    }
    getConfig() {
        return this.options.config;
    }
    getLastPingTimestamp() {
        return this.lastPingTimestamp;
    }
    triggerReload(absoluteFilePath) {
        if (!this.viteServer) {
            return;
        }
        for (const onChange of this.viteServer.watcher.listeners("change")) {
            onChange(absoluteFilePath);
        }
    }
    async renderIndexHtml(originalUrl) {
        if (!this.viteServer) {
            throw new Error(`Vite server is not running.`);
        }
        const template = await fs_extra_1.default.readFile(this.options.shadowHtmlFilePath, "utf-8");
        return await this.viteServer.transformIndexHtml(originalUrl, template);
    }
}
exports.ViteManager = ViteManager;
